R"""


cd ~/Desktop/projects/extract_merge1
export PYTHONPATH=$PYTHONPATH:~/Desktop/projects/extract_merge1


CUDA_VISIBLE_DEVICES=0 python -i local_scripts/ll/hans_ablate_01.py

"""
import dataclasses
from importlib import reload
import itertools
import os
import pickle
import time

import matplotlib.pyplot as plt
import numpy as np
import seaborn as sns
import tensorflow as tf
from tqdm import tqdm
from transformers import AutoTokenizer, TFAutoModelForSequenceClassification

from em import datasets as em_datasets
from em.fishers import diagonal
from em.merging import merging
from em.models import transformer_model_vars as tmv
from em.projects.anli import anli_misc1 as am
from em.projects.ll import hans_util
from em.projects.ll import hans_analysis as ha
from em.projects.wino import nmf_components_fisher as ncf
from em.tools.clustering import vat
from em.projects.ll import hans_labeling
from em.projects.ll import hans_labeling_analysis as hla
from em.util import flat_pack
from em.util import hf_util

from em.util.color_util import cu

###############################################################################

EXPS_DIR = '/fruitbasket/users/m/project_data/extract_merge1/ll1'
MODELS_DIR = os.path.join(EXPS_DIR, 'models')
FISHERS_DIR = os.path.join(EXPS_DIR, 'fishers')
PER_EXAMPLES_FISHERS_DIR = os.path.join(EXPS_DIR, 'per_example_fishers')

###############################################################################

PRETRAINED_MODEL = 'bert-base-uncased'
FROM_PT = True

N_DECOMPS = 25

##########################################################################

FISHER_FILENAME = "feather_berts_{model_number}.hans_lone.validation.h5"

PEF_FILENAME = "feather_berts_{model_number}.hans_lone.no_embeddings.metric_derived.5k.131k.h5"
NMF_FILENAME = "nmf_decomp.v2.per_sub_block.5k.131k.256.{pef}"

###############################################################################


def get_model(model_number: int):
    model = TFAutoModelForSequenceClassification.from_pretrained(
        f'connectivity/feather_berts_{model_number}', from_pt=True)
    model.compile(
        loss=tf.keras.losses.SparseCategoricalCrossentropy(from_logits=True),
        metrics=[tf.keras.metrics.SparseCategoricalAccuracy()],
    )
    return model


def get_mergeable_variables(model):
    variables = hf_util.get_mergeable_variables(model)
    pef_variable_filter = tmv.VariableFilter(merge_embeddings=False)
    return pef_variable_filter.filter_parallel_lists(variables)


def get_variables_by_subset(model):
    variables = get_mergeable_variables(model)
    return tmv.group_by_sub_blocks(variables)


def get_embedding_variables(model):
    variables = hf_util.get_mergeable_variables(model)
    return tmv.group_by_sub_blocks(variables)[0]


def make_container(model_number: int):
    pef_file = PEF_FILENAME.format(model_number=model_number)
    nmf_file = NMF_FILENAME.format(pef=pef_file)
    #
    container = am.load_pef_nmf_analysis_container(
        pef_filepath=os.path.join(PER_EXAMPLES_FISHERS_DIR, pef_file),
        nmf_filepath=os.path.join(PER_EXAMPLES_FISHERS_DIR, nmf_file),
        n_nmfs=N_DECOMPS,
        tokenizer=tokenizer,
        shift_labels=False,
    )
    hans_util.fix_up_hans_container(container)
    container.nmfs.force_load_all()
    #
    return container


def get_fisher(model_number: int):
    filename = FISHER_FILENAME.format(model_number=model_number)
    return diagonal.DiagonalFisher.load(os.path.join(FISHERS_DIR, filename))


##########################################################################

tokenizer = AutoTokenizer.from_pretrained(PRETRAINED_MODEL)

examples = hans_util.get_first_hans_examples(
    'validation',
    5000,
    lambda ds: ds.filter(lambda x: (x['heuristic'] == 'lexical_overlap') & (x['label'] == 1))
)

indicators = hans_labeling.compute_full_indicator(examples, 'ne')

##########################################################################

model_number = 0

container = make_container(model_number)
model = get_model(model_number)


# Some hacking needed to handle the metric derived sparsity.
container.pef.dense_fisher_norms = container.pef.dense_metric_derived_norms

# container.pef.sq_parameter_deltas

##########################################################################


def fix_fisher_md(fisher, container, model, eps: float = 1e-10):
    # packer = flat_pack.FlatPacker([v.shape for v in hf_util.get_mergeable_variables(model)])
    packer = flat_pack.FlatPacker([v.shape for v in get_mergeable_variables(model)])
    sq_deltas = packer.decode_tf(np.maximum(container.pef.sq_parameter_deltas, eps))
    assert len(fisher) == len(sq_deltas)
    return [f / d for f, d in zip(fisher, sq_deltas)]


##########################################################################

ccsp = ncf.SelectionParameters(
    coeff_factor=1 / 3,
    frac_threshold=0.825,
    p_value_threshold=0.04,
)

# ccsp = ncf.SelectionParameters(
#     coeff_factor=0.35,
#     frac_threshold=0.93,
#     p_value_threshold=0.03,
# )
correct_comp_infos = ncf.get_components_appearing_tuned(
    container,
    indicator=container.get_correct_prediction_indicator(),
    selection_parameters=ccsp,
)


# selection_parameters = ncf.SelectionParameters(
#     coeff_factor=1 / 3,
#     frac_threshold=0.825,
#     p_value_threshold=0.04,
# )

selection_parameters = ncf.SelectionParameters(
    coeff_factor=0.35,
    frac_threshold=0.93,
    p_value_threshold=0.03,
)
reload(hla)
tuning_info = hla.compute_hans_tuning_info(container, indicators, selection_parameters)

# example_type = ('subcase_indicators', 'ln_conjunction')
# example_type = ('subcase_indicators', 'ln_subject/object_swap')
example_type = ('subcase_indicators', 'ln_preposition')

# example_type = ('subcase_indicators', 'ln_passive')

# print(cu.hlr('TODO: Only account for correctly predicted components when making the fisher.'))


tcis_by_subset = tuning_info.get_tci_by_nmf(example_type)


def is_correctly_tuned_component(nmf_index: int, component_index: int):
    if not any(tci.component_index == component_index for tci in tcis_by_subset[nmf_index]):
        return False
    if not any(c.nmf_index == nmf_index and c.component_index == component_index for c in correct_comp_infos):
        return False
    return True


fisher_a = tuning_info.make_fisher_for_components(
    get_variables_by_subset(model),
    is_correctly_tuned_component,
)


# fisher_a = tuning_info.make_fisher_for_tuned_components(
#     example_type,
#     get_variables_by_subset(model),
# )
# fisher_b = tuning_info.make_full_fisher(
#     get_variables_by_subset(model),
# )
fisher_b = get_fisher(model_number).fishers

embeddings_zeros = [tf.zeros_like(v) for v in get_embedding_variables(model)]

fisher_a = fix_fisher_md(fisher_a, container, model)
fisher_a = [*embeddings_zeros, *fisher_a]

# fisher_b = fix_fisher_md(fisher_b, container, model)
# fisher_b = [*embeddings_zeros, *fisher_b]

##########################################################################

eval_batch_size = 128

val_ds = em_datasets.load('hans/lexical_overlap_ne', split='validation', sequence_length=64, tokenizer=tokenizer)


def evaluate_model(output_model):
    labels = []
    preds = []
    for x, y in val_ds.batch(eval_batch_size):
        logits = output_model(x, training=False).logits.numpy()
        logits = hans_util.fix_up_hans_logits(logits)
        batch_preds = np.argmax(logits, axis=-1)
        labels.append(y.numpy())
        preds.append(batch_preds)
    labels = np.concatenate(labels, axis=0)
    preds = np.concatenate(preds, axis=0)
    #
    for key, value in indicators.subcase_indicators.items():
        corrects = preds == labels
        acc = corrects[value].astype(np.float64).mean()
        print(f'{key}: {acc}')


##########################################################################

model_a = model

# model_b = get_model(model_number)
# # model_b.set_weights([-0.25 * w for w in model_a.get_weights()])
# model_b.set_weights([-w for w in model_a.get_weights()])
# model_b.bert.embeddings.set_weights(model_a.bert.embeddings.get_weights())
# model_b.bert.pooler.set_weights(model_a.bert.pooler.get_weights())

model_b = TFAutoModelForSequenceClassification.from_pretrained(PRETRAINED_MODEL, from_pt=True)
model_b.bert.embeddings.set_weights(model_a.bert.embeddings.get_weights())

output_model = get_model(model_number)

gen = merging.generate_merged_for_coeffs_set(
    mergeable_models=[model_a, model_b],
    coefficients_set=merging.create_pairwise_grid_coeffs(21),
    # coefficients_set=merging.create_pairwise_grid_coeffs(201)[:21],
    # fishers=[fisher_a, fisher_b],
    fishers=[fisher_b, fisher_a],
    fisher_floor=1e-8,
    favor_target_model=True,
    normalize_fishers=True,
    output_model=output_model,
)
for coefficients, _ in gen:
    print(coefficients)
    evaluate_model(output_model)
    print('')


