R"""


cd ~/Desktop/projects/extract_merge1
export PYTHONPATH=$PYTHONPATH:~/Desktop/projects/extract_merge1


CUDA_VISIBLE_DEVICES= python -i local_scripts/ll/hans_analysis_04.py

"""
import dataclasses
from importlib import reload
import itertools
import os
import pickle
import time

import matplotlib.pyplot as plt
import numpy as np
import seaborn as sns
import tensorflow as tf
from tqdm import tqdm
from transformers import AutoTokenizer, TFAutoModelForSequenceClassification

from em import datasets as em_datasets
from em.models import transformer_model_vars as tmv
from em.projects.anli import anli_misc1 as am
from em.projects.ll import hans_util
from em.projects.ll import hans_analysis as ha
from em.projects.wino import nmf_components_fisher as ncf
from em.tools.clustering import vat
from em.projects.ll import hans_labeling
from em.projects.ll import hans_labeling_analysis as hla
from em.util import hf_util

###############################################################################

EXPS_DIR = '/fruitbasket/users/m/project_data/extract_merge1/ll1'
MODELS_DIR = os.path.join(EXPS_DIR, 'models')
FISHERS_DIR = os.path.join(EXPS_DIR, 'fishers')
PER_EXAMPLES_FISHERS_DIR = os.path.join(EXPS_DIR, 'per_example_fishers')
MISC_DIR = os.path.join(EXPS_DIR, 'misc')

###############################################################################

PRETRAINED_MODEL = 'bert-base-uncased'
FROM_PT = True

N_DECOMPS = 25

##########################################################################

# PEF_FILENAME = "feather_berts_{model_number}.hans.no_embeddings.16k.16k.h5"
# NMF_FILENAME = "nmf_decomp.per_sub_block.16k.16k.256.{pef}"

PEF_FILENAME = "feather_berts_{model_number}.hans_lone.no_embeddings.5k.32k.h5"
NMF_FILENAME = "nmf_decomp.per_sub_block.5k.32k.256.{pef}"

# PEF_FILENAME = "feather_berts_{model_number}.hans_lone.no_embeddings.metric_derived.5k.131k.h5"
# NMF_FILENAME = "nmf_decomp.per_sub_block.5k.131k.256.{pef}"

# PEF_FILENAME = "feather_berts_{model_number}.hans_lone.no_embeddings.metric_derived.5k.131k.h5"
# NMF_FILENAME = "nmf_decomp.v2.per_sub_block.5k.131k.256.{pef}"

###############################################################################


def get_model(model_number: int):
    model = TFAutoModelForSequenceClassification.from_pretrained(
        f'connectivity/feather_berts_{model_number}', from_pt=True)
    model.compile(
        loss=tf.keras.losses.SparseCategoricalCrossentropy(from_logits=True),
        metrics=[tf.keras.metrics.SparseCategoricalAccuracy()],
    )
    return model


def make_container(model_number: int):
    pef_file = PEF_FILENAME.format(model_number=model_number)
    nmf_file = NMF_FILENAME.format(pef=pef_file)
    #
    container = am.load_pef_nmf_analysis_container(
        pef_filepath=os.path.join(PER_EXAMPLES_FISHERS_DIR, pef_file),
        nmf_filepath=os.path.join(PER_EXAMPLES_FISHERS_DIR, nmf_file),
        n_nmfs=N_DECOMPS,
        tokenizer=tokenizer,
        shift_labels=False,
    )
    hans_util.fix_up_hans_container(container)
    container.nmfs.force_load_all()
    #
    return container


##########################################################################

tokenizer = AutoTokenizer.from_pretrained(PRETRAINED_MODEL)

examples = hans_util.get_first_hans_examples(
    'validation',
    5000,
    lambda ds: ds.filter(lambda x: x['heuristic'] == 'lexical_overlap' and x['label'] == 1)
)

indicators = hans_labeling.compute_full_indicator(examples, 'ne')


##########################################################################

container = make_container(0)
model = get_model(0)
# container = make_container(15)

##########################################################################


# selection_parameters = ncf.SelectionParameters(
#     coeff_factor=0.4,
#     frac_threshold=0.6,
#     p_value_threshold=0.04,
# )

selection_parameters = ncf.SelectionParameters(
    coeff_factor=0.485,
    frac_threshold=0.75,
    p_value_threshold=0.03,
)

print('Loaded everything, starting to compute tuning information.')

tuning_info = hla.compute_hans_tuning_info(container, indicators, selection_parameters)

for key, value in tuning_info.iterate_over_tuned_component_infos():
    if len(value) == 0:
        continue
    print(f'{"/".join(key)}: {len(value)}')

print('')
for key, value in indicators.subcase_indicators.items():
    corrects = container.predictions == container.labels
    acc = corrects[value].astype(np.float64).mean()
    print(f'{key}: {acc}')

print('')
for key, value in indicators.template_indicators.items():
    corrects = container.predictions == container.labels
    acc = corrects[value].astype(np.float64).mean()
    print(f'{key}: {acc}')


##########################################################################

# variables = hf_util.get_mergeable_variables(model)

# pef_variable_filter = tmv.VariableFilter(merge_embeddings=False)
# variables = pef_variable_filter.filter_parallel_lists(variables)
# variables_by_subset = tmv.group_by_sub_blocks(variables)


# reload(hla); tuning_info.__class__ = hla.HansTuningInfo
# fisher = tuning_info.make_fisher_for_tuned_components(
#     ('subcase_indicators', 'ln_conjunction'),
#     variables_by_subset
# )


# subcase_indicat
# subcase_indicators/ln_subject/object_swap: 224
# subcase_indicators/ln_preposition: 259
# subcase_indicators/ln_relative_clause: 307
# subcase_indicators/ln_passive: 269
# subcase_indicators/ln_conjunction: 414
