R"""


cd ~/Desktop/projects/extract_merge1
export PYTHONPATH=$PYTHONPATH:~/Desktop/projects/extract_merge1


CUDA_VISIBLE_DEVICES=0 python local_scripts/ll/make_comps_pdf_full_01.py
CUDA_VISIBLE_DEVICES= python local_scripts/ll/make_comps_pdf_full_01.py

"""
from importlib import reload
import os
import time

import numpy as np
from transformers import AutoTokenizer

from em.fishers import per_example
from em.projects.anli import anli_misc1 as am
from em.projects.ll import hans_container
from em.projects.ll import hans_util
from em.tools.nmf import nmf_common

###############################################################################

EXPS_DIR = '/fruitbasket/users/m/project_data/extract_merge1/ll1'
MODELS_DIR = os.path.join(EXPS_DIR, 'models')
FISHERS_DIR = os.path.join(EXPS_DIR, 'fishers')
PER_EXAMPLES_FISHERS_DIR = os.path.join(EXPS_DIR, 'per_example_fishers')

###############################################################################

TOKENIZER = 'bert-base-uncased'

##########################################################################

HANS_LONE_MINI_PEF_FILENAME = "bert_mini_mnli.hans_lone.no_embeddings.5k.131k.h5"
HANS_LONE_MINI_NMF_FILENAME = "nmf_decomp.full.5k.32k.{n_components}.{pef}"

HANS_LONE_PEF_FILENAME = "feather_berts_{model_number}.hans_lone.no_embeddings.5k.32k.h5"
HANS_LONE_NMF_FILENAME = "nmf_decomp.full.5k.32k.{n_components}.{pef}"


##########################################################################

tokenizer = AutoTokenizer.from_pretrained(TOKENIZER)


def make_container_hans_lone_mini(n_components: int):
    pef_file = HANS_LONE_MINI_PEF_FILENAME
    nmf_file = HANS_LONE_MINI_NMF_FILENAME.format(n_components=n_components, pef=pef_file)
    #
    pef = per_example.PerExampleFlatFishers.load(
        os.path.join(PER_EXAMPLES_FISHERS_DIR, pef_file),
        n_examples=None,
        # This leads to the Fishers not being loaded, which ends up being much faster.
        start_fisher_index=0,
        end_fisher_index=0,
    )
    nmf = nmf_common.NmfDecomposition.load(os.path.join(PER_EXAMPLES_FISHERS_DIR, nmf_file))
    nmf.normalize_components_to_unit_norm()
    #
    container = am.PefNmfAnalysisContainer(
        pef=pef,
        nmfs=[nmf],
        tokenizer=tokenizer,
        shift_labels=False,
    )
    hans_util.fix_up_hans_container(container)
    return container


def make_container_hans_lone(model_number: int, n_components: int):
    pef_file = HANS_LONE_PEF_FILENAME.format(model_number=model_number)
    nmf_file = HANS_LONE_NMF_FILENAME.format(n_components=n_components, pef=pef_file)
    #
    pef = per_example.PerExampleFlatFishers.load(
        os.path.join(PER_EXAMPLES_FISHERS_DIR, pef_file),
        n_examples=None,
        # This leads to the Fishers not being loaded, which ends up being much faster.
        start_fisher_index=0,
        end_fisher_index=0,
    )
    nmf = nmf_common.NmfDecomposition.load(os.path.join(PER_EXAMPLES_FISHERS_DIR, nmf_file))
    nmf.normalize_components_to_unit_norm()
    #
    container = am.PefNmfAnalysisContainer(
        pef=pef,
        nmfs=[nmf],
        tokenizer=tokenizer,
        shift_labels=False,
    )
    hans_util.fix_up_hans_container(container)
    return container


def make_container_cuda_test(model_number: int, nmf_file: str):
    pef_file = HANS_LONE_PEF_FILENAME.format(model_number=model_number)
    #
    pef = per_example.PerExampleFlatFishers.load(
        os.path.join(PER_EXAMPLES_FISHERS_DIR, pef_file),
        n_examples=None,
        # This leads to the Fishers not being loaded, which ends up being much faster.
        start_fisher_index=0,
        end_fisher_index=0,
    )
    nmf = nmf_common.NmfDecomposition.load(os.path.join("/fruitbasket/users/m/tmp", f'{nmf_file}.h5'))
    nmf.normalize_components_to_unit_norm()
    #
    container = am.PefNmfAnalysisContainer(
        pef=pef,
        nmfs=[nmf],
        tokenizer=tokenizer,
        shift_labels=False,
    )
    hans_util.fix_up_hans_container(container)
    return container


def make_container_cuda_hans_lone(model_number: int, n_values_pe: int):
    pef_file = HANS_LONE_PEF_FILENAME.format(model_number=model_number)
    nmf_file = f"nmf_decomp.c1024_2kIters_{n_values_pe}pe.feather_berts_{model_number}.hans_lone.all_vars.5k.262144.h5"
    #
    pef = per_example.PerExampleFlatFishers.load(
        os.path.join(PER_EXAMPLES_FISHERS_DIR, pef_file),
        n_examples=None,
        # This leads to the Fishers not being loaded, which ends up being much faster.
        start_fisher_index=0,
        end_fisher_index=0,
    )
    nmf = nmf_common.NmfDecomposition.load(os.path.join(PER_EXAMPLES_FISHERS_DIR, nmf_file))
    nmf.normalize_components_to_unit_norm()
    #
    container = am.PefNmfAnalysisContainer(
        pef=pef,
        nmfs=[nmf],
        tokenizer=tokenizer,
        shift_labels=False,
    )
    hans_util.fix_up_hans_container(container)
    return container


def make_container_cuda_hans_lone_with_flipped(model_number: int, n_values_pe: int):
    pef_file = f"feather_berts_{model_number}.hans_lone_with_flipped.all_vars.10k.131072.h5"
    nmf_file = f"nmf_decomp.c1024_2kIters_{n_values_pe}pe.feather_berts_{model_number}.hans_lone_with_flipped.all_vars.10k.131072.h5"
    #
    pef = per_example.PerExampleFlatFishers.load(
        os.path.join(PER_EXAMPLES_FISHERS_DIR, pef_file),
        n_examples=None,
        # This leads to the Fishers not being loaded, which ends up being much faster.
        start_fisher_index=0,
        end_fisher_index=0,
    )
    nmf = nmf_common.NmfDecomposition.load(os.path.join(PER_EXAMPLES_FISHERS_DIR, nmf_file))
    nmf.normalize_components_to_unit_norm()
    #
    container = am.PefNmfAnalysisContainer(
        pef=pef,
        nmfs=[nmf],
        tokenizer=tokenizer,
        shift_labels=False,
    )
    hans_util.fix_up_hans_container(container)
    return container


##########################################################################

N_EXAMPLES = 12


def make_tex_hans_lone_mini(n_components: int):
    container = make_container_hans_lone_mini(n_components)
    q = container.make_all_components_latex_string(n_examples=N_EXAMPLES)
    with open(f'/fruitbasket/users/m/tmp/hans_lone_mini_{n_components}_all_comps.tex', 'w') as f:
        f.write(container.COMPONENTS_LATEX_FILE_START)
        f.write(q)
        f.write(container.COMPONENTS_LATEX_FILE_END)


def make_tex_hans_lone(model_number: int, n_components: int):
    container = make_container_hans_lone(model_number, n_components)
    q = container.make_all_components_latex_string(n_examples=N_EXAMPLES)
    with open(f'/fruitbasket/users/m/tmp/hans_lone_{model_number}_full_{n_components}_all_comps.tex', 'w') as f:
        f.write(container.COMPONENTS_LATEX_FILE_START)
        f.write(q)
        f.write(container.COMPONENTS_LATEX_FILE_END)


def make_tex_cuda_test(model_number: int, nmf_file: str):
    container = make_container_cuda_test(model_number, nmf_file)
    q = container.make_all_components_latex_string(n_examples=N_EXAMPLES)
    with open(f'/fruitbasket/users/m/tmp/cuda_test_{model_number}_{nmf_file}_all_comps.tex', 'w') as f:
        f.write(container.COMPONENTS_LATEX_FILE_START)
        f.write(q)
        f.write(container.COMPONENTS_LATEX_FILE_END)


def make_tex_cuda_hans_lone(model_number: int, n_values_pe: str):
    print('Loading')
    container = make_container_cuda_hans_lone(model_number, n_values_pe)
    q = container.make_all_components_latex_string(n_examples=N_EXAMPLES)
    with open(f'/fruitbasket/users/m/tmp/cuda_hans_hans_lone_{model_number}_{n_values_pe}_all_comps.tex', 'w') as f:
        f.write(container.COMPONENTS_LATEX_FILE_START)
        f.write(q)
        f.write(container.COMPONENTS_LATEX_FILE_END)
    print('Done')


def make_tex_cuda_hans_lone_with_flipped(model_number: int, n_values_pe: str):
    print('Loading')
    container = make_container_cuda_hans_lone_with_flipped(model_number, n_values_pe)
    q = container.make_all_components_latex_string(n_examples=N_EXAMPLES)
    with open(f'/fruitbasket/users/m/tmp/cuda_hans_hans_lone_with_flipped_{model_number}_{n_values_pe}_all_comps.tex', 'w') as f:
        f.write(container.COMPONENTS_LATEX_FILE_START)
        f.write(q)
        f.write(container.COMPONENTS_LATEX_FILE_END)
    print('Done')


##########################################################################

# make_tex_cuda_test(0, "cuda_nmf_test1")
# make_tex_cuda_test(0, "cuda_nmf_test2")

# make_tex_cuda_hans_lone(0, 65536)
# make_tex_cuda_hans_lone(1, 65536)
# make_tex_cuda_hans_lone(15, 65536)
# make_tex_cuda_hans_lone(25, 65536)

# make_tex_cuda_hans_lone(0, 2 * 65536)

make_tex_cuda_hans_lone_with_flipped(0, 65536)
make_tex_cuda_hans_lone_with_flipped(1, 65536)
make_tex_cuda_hans_lone_with_flipped(15, 65536)
# make_tex_cuda_hans_lone_with_flipped(25, 65536)


# # make_tex_hans_lone_mini(256)
# for model_number in [0, 1, 15, 25]:
#     make_tex_hans_lone(model_number, 256)


R"""

make_pdf() {
    local filename=$1

    rsync -ra -e ssh \
        "m@banana.cs.unc.edu:/fruitbasket/users/m/tmp/${filename}.tex" \
        "$HOME/Downloads/${filename}.tex"

    xelatex -interaction=batchmode -output-directory=/tmp ~/Downloads/${filename}.tex
    mv /tmp/${filename}.pdf ~/Desktop/projects_data/extract_merge1/ll/pdfs
}

make_pdf hans_lone_mini_256_all_comps

make_pdf hans_lone_0_full_256_all_comps
make_pdf hans_lone_1_full_256_all_comps
make_pdf hans_lone_15_full_256_all_comps
make_pdf hans_lone_25_full_256_all_comps

make_pdf hans_lone_25_full_256_all_comps


make_pdf cuda_test_0_cuda_nmf_test1_all_comps
make_pdf cuda_test_0_cuda_nmf_test2_all_comps


make_pdf cuda_hans_hans_lone_0_65536_all_comps
make_pdf cuda_hans_hans_lone_1_65536_all_comps
make_pdf cuda_hans_hans_lone_15_65536_all_comps
make_pdf cuda_hans_hans_lone_25_65536_all_comps

make_pdf cuda_hans_hans_lone_0_131072_all_comps



make_pdf cuda_hans_hans_lone_with_flipped_0_65536_all_comps
make_pdf cuda_hans_hans_lone_with_flipped_1_65536_all_comps
make_pdf cuda_hans_hans_lone_with_flipped_15_65536_all_comps



"""
