R"""


cd ~/Desktop/projects/extract_merge1
export PYTHONPATH=$PYTHONPATH:~/Desktop/projects/extract_merge1


CUDA_VISIBLE_DEVICES=3 python -i local_scripts/pi/scitail_meet_ablate_01.py
"""

from importlib import reload
import random
import os

import matplotlib.pyplot as plt
import seaborn as sns

import numpy as np
import tensorflow as tf
from transformers import AutoTokenizer, TFAutoModelForSequenceClassification

from em import datasets as em_datasets
from em.fishers import diagonal
from em.fishers import per_example
from em.merging import merging
from em.tools.nmf import nmf_common
from em.util import hf_util

from em.projects.anli import anli_misc1 as am
from em.projects.wino import nmf_components_fisher as ncf

from em.projects.pi import binary_ablation_experiment as BAE
from em.projects.pi import qqp_components_context as QCC
from em.projects.pi import qqp_merging_context as QMC
from em.projects.pi import scitail_ablations

EXPS_DIR = '/fruitbasket/users/m/project_data/extract_merge1/pi1'
MODELS_DIR = os.path.join(EXPS_DIR, 'models')
FISHERS_DIR = os.path.join(EXPS_DIR, 'fishers')
PER_EXAMPLES_FISHERS_DIR = os.path.join(EXPS_DIR, 'per_example_fishers')

###############################################################################

MODEL = "textattack/bert-base-uncased-RTE"

PEF_FILENAME = "bert_base_rte.sci_tail_train.all_vars.all_ex.65536.h5"
NMF_FILENAME = f"spH.nmf_decomp.c{1024}_2kIters_{65536}pe_mvpp{8}.{PEF_FILENAME}"

FISHER_FILENAME = "bert_base_rte.scit_tail_final_train.all_vars.all_ex.h5"

##########################################################################

TOKENIZER = 'bert-base-uncased'
tokenizer = AutoTokenizer.from_pretrained(TOKENIZER)

##########################################################################


def get_fisher():
    return diagonal.DiagonalFisher.load(os.path.join(FISHERS_DIR, FISHER_FILENAME))

##########################################################################

# - eval on the train set of scitail, the validation set of scitail, and RTE (probably validation set)
# - multiply/geometric average for intersection, add/average for union.
# - maybe try ablation of HANs component and transfer to sci_tail?


hacc = QCC.QqpComponentContext(
    model_name_pattern=MODEL,
    pef_filepath_pattern=os.path.join(PER_EXAMPLES_FISHERS_DIR, PEF_FILENAME),
    nmf_filepath_pattern=os.path.join(PER_EXAMPLES_FISHERS_DIR, NMF_FILENAME),
    tokenizer=tokenizer,
)
mc = hacc.make_model_context('a')

##########################################################################

N_GEN_EVAL = 3000

gen_eval_ctx = scitail_ablations.get_scitail_eval_ctx(mc, N_GEN_EVAL)
rte_eval_ctx = scitail_ablations.get_rte_eval_ctx(mc)

##########################################################################
nmf = mc.nmf
W = nmf.W
##########################################################################

EX_INDEX = 6969

example = mc.container.examples[EX_INDEX]

# plt.plot(W[EX_INDEX]); plt.show()

top_ex_comps = np.argsort(-W[EX_INDEX])
top_ex_coeffs = W[EX_INDEX][top_ex_comps]

##########################################################################


exp_config = BAE.AblationExperimentConfig(
    component_index=top_ex_comps[0],
    n_coefficients=21,
)
exp = BAE.AblationExperiment(
    config=exp_config,
    mc=mc,
    dense_fisher=get_fisher().fishers,
    extra_eval_contexts=[],
)

##########################################################################

N_SAME_EVAL = 3000
eval_inds = np.arange(N_SAME_EVAL)

# sg_ds = exp.sign_guider.get_ds_for_examples(exp.sorted_example_indices[EX_INDEX:EX_INDEX + 1])
sg_ds = exp.sign_guider.get_ds_for_examples([EX_INDEX])
sign_guide = exp.sign_guider.compute_loss_gradient(
    sg_ds.batch(exp_config.compute_gradient_batch_size),
    normalize_gradients_by_example=False,
    # normalize_gradients_by_example=True,
)


def run_ablation(ablating_fisher, delta):
    gen = exp.stream_merge_based_ablation(
        retaining_fisher=exp.dense_fisher,
        #
        ablating_fisher=ablating_fisher,
        #
        sign_guide=sign_guide,
        delta=delta,
    )
    scitail_val_kls = []
    example_kls = []
    for coefficients, output_model in gen:
        gen_eval_results = gen_eval_ctx.evaluate(output_model)
        # sub_eval_results = exp.eval_context.evaluate(output_model, exp.sorted_example_indices[EX_INDEX:EX_INDEX + 1])
        sub_eval_results = exp.eval_context.evaluate(output_model, [EX_INDEX])
        #
        scitail_val_kls.append(gen_eval_results.kl())
        example_kls.append(sub_eval_results.kl())
        print(coefficients)
        print(f'  {gen_eval_results.kl()}, {gen_eval_results.acc():.4f}, {gen_eval_results.loss()}')
        print(f'  {sub_eval_results.kl()}, {sub_eval_results.acc()}')
        print('')
        #
    return scitail_val_kls, example_kls




scitail_val_kls0, example_kls0 = run_ablation(
    diagonal.compute_fisher_for_model(mc.model, sg_ds.batch(4), variables=mc.variables),
    delta=3e-6,
)

scitail_val_kls1, example_kls1 = run_ablation(
    mc.make_fisher_for_components(top_ex_comps[:1]),
    delta=2e-4,
)

scitail_val_kls2, example_kls2 = run_ablation(
    mc.make_fisher_for_components(top_ex_comps[:2]),
    delta=1e-4,
)

scitail_val_kls5, example_kls5 = run_ablation(
    mc.make_fisher_for_components(top_ex_comps[:5]),
    delta=1e-4,
)

scitail_val_kls10, example_kls10 = run_ablation(
    mc.make_fisher_for_components(top_ex_comps[:10]),
    delta=1e-4,
)


plt.plot(scitail_val_kls0, example_kls0, label='pef')
plt.plot(scitail_val_kls1, example_kls1, label='1 comp')
plt.plot(scitail_val_kls2, example_kls2, label='2 comp')
plt.plot(scitail_val_kls5, example_kls5, label='5 comp')
plt.plot(scitail_val_kls10, example_kls10, label='10 comp')
plt.legend(loc='best')
plt.show()

#

reload(scitail_ablations)

st_kls1, ex_kls1 = run_ablation(
    scitail_ablations.make_fisher_linear_combination(mc, top_ex_comps[:1], top_ex_coeffs[:1]),
    delta=2e-4,
)

st_kls2, ex_kls2 = run_ablation(
    scitail_ablations.make_fisher_linear_combination(mc, top_ex_comps[:2], top_ex_coeffs[:2]),
    delta=1e-4,
)

st_kls5, ex_kls5 = run_ablation(
    scitail_ablations.make_fisher_linear_combination(mc, top_ex_comps[:5], top_ex_coeffs[:5]),
    delta=7e-5,
)

st_kls10, ex_kls10 = run_ablation(
    scitail_ablations.make_fisher_linear_combination(mc, top_ex_comps[:10], top_ex_coeffs[:10]),
    delta=3e-5,
)

st_kls100, ex_kls100 = run_ablation(
    scitail_ablations.make_fisher_linear_combination(mc, top_ex_comps[:100], top_ex_coeffs[:100]),
    delta=4e-5,
)


plt.plot(scitail_val_kls0, example_kls0, label='pef')
plt.plot(st_kls1, ex_kls1, label='1 comp')
plt.plot(st_kls2, ex_kls2, label='2 comp')
plt.plot(st_kls5, ex_kls5, label='5 comp')
plt.plot(st_kls10, ex_kls10, label='10 comp')
plt.plot(st_kls100, ex_kls100, label='100 comp')
plt.legend(loc='best')
plt.show()

#

reload(scitail_ablations)

st_kls_gm2, ex_kls_gm2 = run_ablation(
    scitail_ablations.make_fisher_geometric_mean(mc, top_ex_comps[:2], top_ex_coeffs[:2], fisher_floor=1e-7),
    delta=4e-4,
)

st_kls_gm3, ex_kls_gm3 = run_ablation(
    scitail_ablations.make_fisher_geometric_mean(mc, top_ex_comps[:3], top_ex_coeffs[:3], fisher_floor=1e-7),
    delta=1e-3,
)

st_kls_gm5, ex_kls_gm5 = run_ablation(
    scitail_ablations.make_fisher_geometric_mean(mc, top_ex_comps[:5], top_ex_coeffs[:5], fisher_floor=1e-7),
    delta=4e-3,
)

plt.plot(scitail_val_kls0, example_kls0, label='pef')
plt.plot(st_kls1, ex_kls1, label='1 comp')
plt.plot(st_kls_gm2, ex_kls_gm2, label='2 comp')
plt.plot(st_kls_gm3, ex_kls_gm3, label='3 comp')
plt.plot(st_kls_gm5, ex_kls_gm5, label='5 comp')
plt.plot(st_kls2, ex_kls2, label='2 comp')
plt.plot(st_kls5, ex_kls5, label='5 comp')
plt.legend(loc='best')
plt.show()

#


#
