R"""


cd ~/Desktop/projects/extract_merge1
export PYTHONPATH=$PYTHONPATH:~/Desktop/projects/extract_merge1


CUDA_VISIBLE_DEVICES=2 python -i local_scripts/transfer1/anli1/anli_per_sublock_nmf_dev001.py

"""
from importlib import reload
import os
import time

import matplotlib.pyplot as plt
import numpy as np
import scipy.sparse as sps
import seaborn as sns
import tensorflow as tf
from transformers import TFAutoModelForSequenceClassification, AutoTokenizer

from em import datasets as em_datasets
from em.fishers import diagonal
from em.fishers import per_example
from em.merging import merging
from em.models import transformer_model_vars as tmv
from em.tools.nmf import nmf_common
from em.tools.nmf import nmf_transform
from em.util import flat_pack
from em.util import hf_util

rocket_cmap = sns.color_palette("rocket", as_cmap=True)

###############################################################################

EXPS_DIR = '/fruitbasket/users/m/project_data/extract_merge1/anli_correct1'
MODELS_DIR = os.path.join(EXPS_DIR, 'models')
FISHERS_DIR = os.path.join(EXPS_DIR, 'fishers')
PER_EXAMPLES_FISHERS_DIR = os.path.join(EXPS_DIR, 'per_example_fishers')

###############################################################################

MODEL = "textattack/bert-base-uncased-MNLI"
FROM_PT = True

PRETRAINED_MODEL = 'bert-base-uncased'

PER_EXAMPLES_FISHERS = "textattack_bert_base_uncased_MNLI.anli_r3.no_embeddings.sparse_dynamic_raw.32k.32k.h5"
DECOMP = f"nmf_decomp.per_sub_block.16k.16k.256.{PER_EXAMPLES_FISHERS}"

###############################################################################

N_EXAMPLES = 16 * 1024
START_FISHER_INDEX, END_FISHER_INDEX = (0, 16 * 1024)

model = TFAutoModelForSequenceClassification.from_pretrained(
    MODEL,
    from_pt=FROM_PT,
)
tokenizer = AutoTokenizer.from_pretrained(PRETRAINED_MODEL)

print('Starting to load saved per-example Fishers.')
start = time.time()
pe_fishers_data = per_example.PerExampleFlatFishers.load(
    os.path.join(PER_EXAMPLES_FISHERS_DIR, PER_EXAMPLES_FISHERS),
    n_examples=N_EXAMPLES,
    start_fisher_index=START_FISHER_INDEX,
    end_fisher_index=END_FISHER_INDEX,
)
print('Load saved per-example Fishers time: ', time.time() - start)


###############################################################################

def load_nmf_decomp(subset_index: int):
    print('Starting to load saved NMF decomposition.')
    start = time.time()
    filename = f"{DECOMP[:-3]}.ssi{subset_index}.h5"
    decomp = nmf_common.NmfDecomposition.load(os.path.join(PER_EXAMPLES_FISHERS_DIR, filename))
    decomp.normalize_components_to_unit_norm()
    # decomp.H = decomp.get_full_H()
    print('Load saved NMF decomposition time: ', time.time() - start)
    return decomp


decomp1 = load_nmf_decomp(20)
decomp2 = load_nmf_decomp(21)

###############################################################################


def print_top_examples(decomp, component: int, n_examples: int):
    _, inds = tf.math.top_k(decomp.W[:, component], k=n_examples)
    for ind in inds:
        label = pe_fishers_data.labels[ind]
        if isinstance(label, tf.Tensor):
            label = label.numpy()
        #
        # Stuff for MNLI:
        label = (label + 1) % 3
        #
        pred = np.argmax(pe_fishers_data.predicted_logits[ind])
        example = tokenizer.decode(pe_fishers_data.input_ids[ind])
        example = example.replace(tokenizer.pad_token, '')
        example = example.strip()
        print(f'{label}, {pred}: {example}')


print_top_examples(decomp1, n_examples=8, component=0)
print_top_examples(decomp2, n_examples=8, component=0)
