R"""


cd ~/Desktop/projects/extract_merge1
export PYTHONPATH=$PYTHONPATH:~/Desktop/projects/extract_merge1


CUDA_VISIBLE_DEVICES=0 python -i local_scripts/transfer1/anli1/anli_scores001.py

"""
import dataclasses
from importlib import reload
import os
import time

from colorama import Fore, Back, Style
import matplotlib.pyplot as plt
import numpy as np
import scipy.sparse as sps
import seaborn as sns
import tensorflow as tf
from transformers import TFAutoModelForSequenceClassification, AutoTokenizer

from em import datasets as em_datasets
from em.fishers import diagonal
from em.fishers import per_example
from em.merging import merging
from em.models import transformer_model_vars as tmv
from em.tools.nmf import nmf_common
from em.tools.nmf import nmf_transform
from em.util import flat_pack
from em.util import hf_util
from em.util.color_util import cu

from em.projects.anli import anli_misc1 as am


rocket_cmap = sns.color_palette("rocket", as_cmap=True)

###############################################################################

EXPS_DIR = '/fruitbasket/users/m/project_data/extract_merge1/anli_correct1'
MODELS_DIR = os.path.join(EXPS_DIR, 'models')
FISHERS_DIR = os.path.join(EXPS_DIR, 'fishers')
PER_EXAMPLES_FISHERS_DIR = os.path.join(EXPS_DIR, 'per_example_fishers')

###############################################################################

SEQUENCE_LENGTH = 128
BATCH_SIZE = 256

MODEL = "textattack/bert-base-uncased-MNLI"
FROM_PT = True

PRETRAINED_MODEL = 'bert-base-uncased'

###############################################################################

tokenizer = AutoTokenizer.from_pretrained(PRETRAINED_MODEL)

model = TFAutoModelForSequenceClassification.from_pretrained(
    MODEL,
    from_pt=FROM_PT,
)
model.compile(
    metrics=tf.keras.metrics.SparseCategoricalAccuracy(),
    loss=tf.keras.losses.SparseCategoricalCrossentropy(from_logits=True),
)

############################################

mnli_train = em_datasets.load(
    'glue/mnli',
    split='train',
    tokenizer=tokenizer,
    sequence_length=128,
)
mnli_train = em_datasets.glue.fix_text_attack_mnli_labeling(mnli_train)
mnli_val = em_datasets.load(
    'glue/mnli',
    split='validation',
    tokenizer=tokenizer,
    sequence_length=128,
)
mnli_val = em_datasets.glue.fix_text_attack_mnli_labeling(mnli_val)

anli_train = em_datasets.load(
    'anli/r3',
    split='train',
    tokenizer=tokenizer,
    sequence_length=128,
)
anli_train = em_datasets.glue.fix_text_attack_mnli_labeling(anli_train)
anli_val = em_datasets.load(
    'anli/r3',
    split='validation',
    tokenizer=tokenizer,
    sequence_length=128,
)
anli_val = em_datasets.glue.fix_text_attack_mnli_labeling(anli_val)

############################################

N_EXAMPLES = 4096

model.evaluate(mnli_train.take(N_EXAMPLES).batch(BATCH_SIZE))
model.evaluate(mnli_val.take(N_EXAMPLES).batch(BATCH_SIZE))

model.evaluate(anli_train.take(N_EXAMPLES).batch(BATCH_SIZE))
model.evaluate(anli_val.take(N_EXAMPLES).batch(BATCH_SIZE))


"""
MNLI train: loss: 0.3317 - sparse_categorical_accuracy: 0.8708
MNLI validation: loss: 0.5257 - sparse_categorical_accuracy: 0.8149

ANLI r3 train: loss: 0.8497 - sparse_categorical_accuracy: 0.7092
ANLI r3 validation: loss: 2.1166 - sparse_categorical_accuracy: 0.3175

ANLI r2 train: loss: 0.9041 - sparse_categorical_accuracy: 0.6960
ANLI r2 validation: loss: 2.4810 - sparse_categorical_accuracy: 0.2840

ANLI r1 train: loss: 0.8430 - sparse_categorical_accuracy: 0.7148
ANLI r1 validation: loss: 2.4119 - sparse_categorical_accuracy: 0.2390


"""
