R"""


cd ~/Desktop/projects/extract_merge1
export PYTHONPATH=$PYTHONPATH:~/Desktop/projects/extract_merge1


CUDA_VISIBLE_DEVICES=0 python -i local_scripts/transfer1/transfer_dev001.py

"""
from importlib import reload
import os
import time

import matplotlib.pyplot as plt
import numpy as np
import tensorflow as tf
from transformers import TFAutoModelForSequenceClassification, AutoTokenizer

from em import datasets as em_datasets
from em.fishers import diagonal
from em.fishers import per_example
from em.merging import merging
from em.models import transformer_model_vars as tmv
from em.tools.nmf import nmf_common
from em.util import flat_pack
from em.util import hf_util

###############################################################################

EXPS_DIR = '/fruitbasket/users/m/project_data/extract_merge1/transfer1'
MODELS_DIR = os.path.join(EXPS_DIR, 'models')
FISHERS_DIR = os.path.join(EXPS_DIR, 'fishers')
PER_EXAMPLES_FISHERS_DIR = os.path.join(EXPS_DIR, 'per_example_fishers')

###############################################################################

PRETRAINED_MODEL = "bert-base-uncased"

MODEL = "frozen_bert_base_rte_001"
FROM_PT = False

TASK = "glue/rte"

SEQUENCE_LENGTH = 128

FISHER = f'{MODEL}.fisher.h5'

###############################################################################
# TODO: In proper code, I can probably multithread/multiprocess this to do all these
# loads below in parallel.
#############################

tokenizer = AutoTokenizer.from_pretrained(PRETRAINED_MODEL)

print('Starting to load fisher.')
start = time.time()
dense_fisher = diagonal.DiagonalFisher.load(os.path.join(FISHERS_DIR, FISHER))
print('Load saved fishertime: ', time.time() - start)

###########################################################

frozen_model = TFAutoModelForSequenceClassification.from_pretrained(
    os.path.join(MODELS_DIR, MODEL), from_pt=False)

frozen_model.compile(
    metrics=tf.keras.metrics.SparseCategoricalAccuracy(),
    loss=tf.keras.losses.SparseCategoricalCrossentropy(from_logits=True),
)


###########################################################

train_ds = em_datasets.load(
    TASK,
    split='train',
    tokenizer=tokenizer,
    sequence_length=SEQUENCE_LENGTH,
)
# train_ds = train_ds.batch()

frozen_model.evaluate(train_ds.batch(256))

###########################################################

model = TFAutoModelForSequenceClassification.from_pretrained(
    "textattack/bert-base-uncased-MNLI", from_pt=True)

model.compile(
    metrics=tf.keras.metrics.SparseCategoricalAccuracy(),
    loss=tf.keras.losses.SparseCategoricalCrossentropy(from_logits=True),
)

ds = em_datasets.load(
    'glue/mnli',
    split='train',
    tokenizer=tokenizer,
    sequence_length=SEQUENCE_LENGTH,
)
ds = em_datasets.glue.fix_text_attack_mnli_labeling(ds)
model.evaluate(ds.take(16 * 1024).batch(256))

# frozen_model.bert.embeddings.set_weights(model.bert.embeddings.get_weights())
# frozen_model.evaluate(train_ds.batch(256))
