R"""

Raw sparse fishers.



cd ~/Desktop/projects/extract_merge1
export PYTHONPATH=$PYTHONPATH:~/Desktop/projects/extract_merge1


CUDA_VISIBLE_DEVICES=0 python -i local_scripts/transfer1/transfer_dev002.py

"""
from importlib import reload
import os
import time

import matplotlib.pyplot as plt
import numpy as np
import tensorflow as tf
from transformers import TFAutoModelForSequenceClassification, AutoTokenizer

from em import datasets as em_datasets
from em.fishers import diagonal
from em.fishers import per_example
from em.merging import merging
from em.models import transformer_model_vars as tmv
from em.tools.nmf import nmf_common
from em.util import flat_pack
from em.util import hf_util

###############################################################################

EXPS_DIR = '/fruitbasket/users/m/project_data/extract_merge1/transfer1'
MODELS_DIR = os.path.join(EXPS_DIR, 'models')
FISHERS_DIR = os.path.join(EXPS_DIR, 'fishers')
PER_EXAMPLES_FISHERS_DIR = os.path.join(EXPS_DIR, 'per_example_fishers')

###############################################################################

PRETRAINED_MODEL = "bert-base-uncased"

MODEL = "frozen_bert_base_rte_001"
FROM_PT = False

TASK = "glue/rte"

SEQUENCE_LENGTH = 128

FISHER = f'{MODEL}.fisher.h5'
PER_EXAMPLES_FISHERS = "bert_base_mnli.sparse_dynamic_raw.32k.16k.h5"
DECOMP = f"nmf_decomp.16k.8k.64.reduced_1.{PER_EXAMPLES_FISHERS}"

N_EXAMPLES = 16 * 1024
START_FISHER_INDEX, END_FISHER_INDEX = (0, 8 * 1024)


###############################################################################
# TODO: In proper code, I can probably multithread/multiprocess this to do all these
# loads below in parallel.
#############################

tokenizer = AutoTokenizer.from_pretrained(PRETRAINED_MODEL)

print('Starting to load fisher.')
start = time.time()
dense_fisher = diagonal.DiagonalFisher.load(os.path.join(FISHERS_DIR, FISHER))
print('Load saved fishertime: ', time.time() - start)

print('Starting to load saved NMF decomposition.')
start = time.time()
decomp = nmf_common.NmfDecomposition.load(os.path.join(PER_EXAMPLES_FISHERS_DIR, DECOMP))
decomp.normalize_components_to_unit_norm()
print('Load saved NMF decomposition time: ', time.time() - start)

print('Starting to load saved per-example Fishers.')
start = time.time()
pe_fishers_data = per_example.PerExampleFlatFishers.load(
    os.path.join(PER_EXAMPLES_FISHERS_DIR, PER_EXAMPLES_FISHERS),
    n_examples=N_EXAMPLES,
    start_fisher_index=START_FISHER_INDEX,
    end_fisher_index=END_FISHER_INDEX,
)
print('Load saved per-example Fishers time: ', time.time() - start)


###########################################################

frozen_model = TFAutoModelForSequenceClassification.from_pretrained(
    os.path.join(MODELS_DIR, MODEL), from_pt=False)

frozen_model.compile(
    metrics=tf.keras.metrics.SparseCategoricalAccuracy(),
    loss=tf.keras.losses.SparseCategoricalCrossentropy(from_logits=True),
)

###########################################################


def sp_de_dot_product(sp, de):
    assert len(sp.shape) == len(de.shape) == 1
    dp = tf.sparse.sparse_dense_matmul(
        tf.sparse.reshape(sp, [1, sp.shape[0]]),
        de[:, None]
    )
    return tf.reshape(dp, []).numpy()


frozen_fisher = tf.linalg.l2_normalize(dense_fisher.as_flat_fisher())
sparse_Hs = decomp.get_full_sparse_H()

dot_prods = []
for h in sparse_Hs:
    dp = sp_de_dot_product(h, frozen_fisher)
    dot_prods.append(dp)

plt.plot(dot_prods);plt.show()


###########################################################

def print_top_examples(component: int, n_examples: int):
    _, inds = tf.math.top_k(decomp.W[:, component], k=n_examples)
    for ind in inds:
        label = pe_fishers_data.labels[ind]
        if isinstance(label, tf.Tensor):
            label = label.numpy()
        # Stuff for MNLI:
        label = (label + 1) % 3
        #
        pred = np.argmax(pe_fishers_data.predicted_logits[ind])
        example = tokenizer.decode(pe_fishers_data.input_ids[ind])
        example = example.replace(tokenizer.pad_token, '')
        # example = example.replace(tokenizer.bos_token, '')
        # example = example.replace(tokenizer.eos_token, '')
        example = example.strip()
        print(f'{label}, {pred}: {example}')


lowest_dot_prods = np.argsort(dot_prods)
highest_dot_prods = np.argsort(-dot_prods)

print_top_examples(lowest_dot_prods[0], n_examples=16)
print_top_examples(lowest_dot_prods[1], n_examples=16)
print_top_examples(lowest_dot_prods[2], n_examples=16)
print_top_examples(lowest_dot_prods[3], n_examples=16)
print_top_examples(lowest_dot_prods[-1], n_examples=16)
