R"""Writes a bunch of random expressions to a file.


cd ~/Desktop/projects/extract_merge1
export PYTHONPATH=$PYTHONPATH:~/Desktop/projects/extract_merge1

python3 scripts1/data_gen/antiderivative/generate_expressions.py \
    --output_file=/tmp/test_expressions \
    --n_examples=256 \
    --n_processes=1


"""
import multiprocessing as mp
import os

from absl import app
from absl import flags

from tqdm import tqdm

from em.datasets.antiderivative import expressions
from em.util.color_util import cu

FLAGS = flags.FLAGS

flags.DEFINE_string("output_file", None, "Path to file to write expressions to.")

flags.DEFINE_integer("n_examples", None, 'The number of examples to generate.')

# TODO: Possibly allow configuration of other generation parameters.
flags.DEFINE_integer("max_ops", 10, '')
flags.DEFINE_integer("seconds_per_attempt", 1, '')

flags.DEFINE_bool("append_to_file", True, '')

flags.DEFINE_integer("flush_every", 50, '')

flags.DEFINE_integer("n_processes", 1, '')


class GeneratorProcess(mp.Process):

    def __init__(self, output_queue: mp.Queue, max_ops: int, seconds_per_attempt: int):
        super().__init__()
        self.output_queue = output_queue
        self.seconds_per_attempt = seconds_per_attempt
        self.gen = expressions.ExpressionGenerator(
            max_ops=max_ops,
        )

    def run(self):
        while True:
            try:
                expr = self.gen.maybe_generate_expression(self.seconds_per_attempt)
            except Exception as e:
                print(cu.hlr(e))
                continue

            if expr is None:
                continue
            self.output_queue.put(str(expr))


def main_single_process():
    gen = expressions.ExpressionGenerator(
        max_ops=FLAGS.max_ops,
    )

    output_file = os.path.expanduser(FLAGS.output_file)
    with open(output_file, 'a' if FLAGS.append_to_file else 'w') as f:
        for i in tqdm(range(FLAGS.n_examples)):
            try:
                expr = gen.generate_expression(FLAGS.seconds_per_attempt)
            except Exception as e:
                print(cu.hlr(e))
                continue
            f.write(f'{str(expr)}\n')
            if ((i + 1) % FLAGS.flush_every) == 0:
                f.flush()
                os.fsync(f)


def main(_):
    assert FLAGS.n_processes == 1, 'TODO: Timeouts and multiprocessing are not playing nice together.'

    if FLAGS.n_processes == 1:
        return main_single_process()

    # # queue_size = 100 * FLAGS.n_processes
    # # output_queue = mp.Queue(queue_size)
    # output_queue = mp.Queue()

    # processes = [
    #     GeneratorProcess(
    #         output_queue=output_queue,
    #         max_ops=FLAGS.max_ops,
    #         seconds_per_attempt=FLAGS.seconds_per_attempt,
    #     )
    #     for _ in range(FLAGS.n_processes)
    # ]
    # for p in processes:
    #     p.start()

    # output_file = os.path.expanduser(FLAGS.output_file)
    # with open(output_file, 'a' if FLAGS.append_to_file else 'w') as f:
    #     for i in tqdm(range(FLAGS.n_examples)):
    #         expr = output_queue.get()
    #         f.write(f'{expr}\n')
    #         if ((i + 1) % FLAGS.flush_every) == 0:
    #             f.flush()
    #             os.fsync()

    # for p in processes:
    #     p.terminate()


if __name__ == "__main__":
    app.run(main)
